/******************************************************************************
Description:

  Error/status code class

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Comm-C++-Current/library/CIF_Comm_Status.H,v $
  $Date: 1998/09/28 20:16:24 $
  $Revision: 1.4 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

#if !defined(CIF_INCLUDE_COMM_STATUS)
#define CIF_INCLUDE_COMM_STATUS 1

class CIF_Comm_Status
{
    public:
    enum Status
    {
	/* The operation was successful */
	SUCCESS = 0,

	/* The operation failed */
	FAILURE = 1,

	/* The operation failed because the object had already been activated
	   by some previous operation (ex: the connection had already been
	   established) */
	OBJECT_ACTIVE = 2,
    
	/* The operation failed because some previous operation deactived the
	   object (ex: the connection was closed in a previous operation, so
	   any subsequent sends will fail) */
	OBJECT_INACTIVE = 3,

	/* The object being accessed has been previously deleted */
	OBJECT_DELETED = 4,

	/* An out of memory condition occurred */
	INSUFFICIENT_MEMORY = 5,

	/* The format of the specified URL was not valid */
	URL_FORMAT_BAD = 6,
    
	/* The scheme of the specified URL is not supported */
	URL_SCHEME_UNSUPPORTED = 7,

	/* The allotted amount of time has elapsed and the operation has yet to
           successfully complete */
	TIMED_OUT = 8,

	/* The requested operation would block which violates an assumption
	   made by the library implementation */
	WOULD_BLOCK = 9,

	/* An internal error occurred (shouldn't happen).  Please contact the
	   author. */
	INTERNAL_ERROR = -1
    };
    

# if defined(CIF_USE_INLINES)
    public: inline
    CIF_Comm_Status()
    : l_status(SUCCESS)
    {
    }

    public: inline
    CIF_Comm_Status(
	const CIF_Comm_Status &		status)
    : l_status(status.l_status)
    {
    }

    public: inline
    CIF_Comm_Status(
	const int			status)
    : l_status(status)
    {
    }
    
    public: inline
    CIF_Comm_Status &
    operator =(
	const int			status)
    {
	l_status = status;
	return *this;
    }
    
    public: inline
    CIF_Comm_Status &
    operator =(
	const CIF_Comm_Status &		status)
    {
	l_status = status.l_status;
	return *this;
    }
    
    public: inline
    operator int() const
    {
	return l_status;
    }
# else
    public:
    CIF_Comm_Status();

    public:
    CIF_Comm_Status(
	const CIF_Comm_Status &		status);
    
    public:
    CIF_Comm_Status(
	const int			status);
    
    public:
    CIF_Comm_Status &
    operator =(
	const int			status);

    public:
    CIF_Comm_Status &
    operator =(
	const CIF_Comm_Status &		status);
    
    public:
    operator int() const;
# endif
    
    public:
    const char *
    get_Description();

    /*
     * TODO: (THREAD-SAFETY) l_status needs to be changed to thread specific
     * data, but that doesn't seem to be completely supported by the
     * non-threaded version of Globus right now
     */
    private:
    int					l_status;
};

#endif /* CIF_INCLUDE_COMM_STATUS */
