/******************************************************************************
Description:

  Connection class for Nexus using UUM (unreliable unordered multicast)

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Comm-C++-Current/library/CIF_I_Comm_Connection_UUM.C,v $
  $Date: 1998/11/05 20:50:02 $
  $Revision: 1.9 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_I_Comm_Connection_UUM.H"

#include <string.h>


/*
 * CIF_I_Comm_Connection_UUM::open_Connection()
 */
CIF_I_Comm_Connection::Connection_Flow_Mask
CIF_I_Comm_Connection_UUM::open_Connection(
    ARG_IN const char *				url,
    ARG_INMOD
    CIF_I_Comm_Endpoint::Properties &		ep_properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    globus_nexus_proto_info_udp_t		proto_info;
    globus_url_t				url_info;

    if (globus_url_parse((char *) url, &url_info) != GLOBUS_SUCCESS)
    {
	CIF_Comm::status = CIF_Comm_Status::URL_FORMAT_BAD;
	return CONNECTION_UNESTABLISHED;
    }

    if (url_info.scheme == GLOBUS_NULL ||
	strcmp(url_info.scheme, CIF_COMM_PROTOCOL_UUM_STR) != 0)
    {
	globus_url_destroy(&url_info);

	CIF_Comm::status = CIF_Comm_Status::URL_SCHEME_UNSUPPORTED;
	return CONNECTION_UNESTABLISHED;
    }

    int						tmp_int;

    globus_nexus_proto_info_udp_init(&proto_info);
    
    proto_info.host = url_info.host;
    proto_info.port = url_info.port;
    
    if (url_info.url_path != GLOBUS_NULL &&
	strcmp(url_info.url_path, "/") != 0)
    {
	int					options_len;
	char *					options_str;

	options_len = strlen(url_info.url_path) + 2;
	options_str = (char *) malloc(options_len);

	strcpy(options_str, url_info.url_path);
	options_str[0] = '+';
	options_str[options_len - 2] = '+';
	options_str[options_len - 1] = '\0';
	
	while(options_str != GLOBUS_NULL && strlen(options_str) > 1)
	{
	    if (sscanf(options_str, "+ttl=%d+", &tmp_int) == 1)
	    {
		proto_info.ttl = tmp_int;
	    }
	    else if (strncmp(options_str,
			     "+loopback=true+",
			     strlen("+loopback=true+")) == 0)
	    {
		proto_info.loopback = GLOBUS_TRUE;
	    }
	    else if (strncmp(options_str,
			     "+loopback=false+",
			     strlen("+loopback=false+")) == 0)
	    {
		proto_info.loopback = GLOBUS_FALSE;
	    }
	    else if (strncmp(options_str,
			     "+multiple_senders=true+",
			     strlen("+multiple_senders=true+")) == 0)
	    {
		proto_info.multi_sender = GLOBUS_TRUE;
	    }
	    else if (strncmp(options_str,
			     "+multiple_senders=false+",
			     strlen("+multiple_senders=false+")) == 0)
	    {
		proto_info.multi_sender = GLOBUS_FALSE;
	    }
	    else
	    {
		free(options_str);
		
		CIF_Comm::status = CIF_Comm_Status::URL_FORMAT_BAD;
		return CONNECTION_UNESTABLISHED;
	    }

	    options_str = strpbrk(options_str + 1, "+");
	}
    }

    /*
     * Create endpoint and startpoint
     */
    ep_properties.set_Protocol(GLOBUS_NEXUS_PROTO_TYPE_UDP,
			       &proto_info,
			       sizeof(proto_info));
    
    if (l_ep.activate(ep_properties) == false)
    {
	globus_url_destroy(&url_info);

	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return CONNECTION_UNESTABLISHED;
    }

    globus_url_destroy(&url_info);
    
    if (l_sp.activate(l_ep) == false)
    {
	l_ep.deactivate();
	
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return CONNECTION_UNESTABLISHED;
    }

    CIF_Comm::status = CIF_Comm_Status::SUCCESS;
    return CONNECTION_BIDIRECTIONAL;
}
/* CIF_I_Comm_Connection::open_Connection() */


/*
 * CIF_I_Comm_Connection_UUM::open_Connection()
 */
CIF_I_Comm_Connection::Connection_Flow_Mask
CIF_I_Comm_Connection_UUM::open_Connection(
    ARG_INOUT globus_nexus_buffer_t &		nx_buf,
    ARG_INMOD
    CIF_I_Comm_Endpoint::Properties &		ep_properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    CIF_Comm::status = CIF_Comm_Status::FAILURE;
    return CONNECTION_UNESTABLISHED;
}
/* CIF_I_Comm_Connection_UUM::open_Connection() */
