/******************************************************************************
CIF_I_Comm_Startpoint.C

Description:

  Classes to provide simple Nexus startpoint functionality

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Comm-C++-Current/library/CIF_I_Comm_Startpoint.C,v $
  $Date: 1998/09/28 10:43:35 $
  $Revision: 1.6 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_I_Comm_Startpoint.H"

#include <string.h>


/******************************************************************************
			   CIF_I_Comm_Startpoint methods
******************************************************************************/


/*
 * CIF_I_Comm_Startpoint::CIF_I_Comm_Startpoint()
 */
CIF_I_Comm_Startpoint::CIF_I_Comm_Startpoint()
{
    /*
     * Set state to indicate that the object has been successfully initialized
     */
    l_object_state = OBJECT_STATE_INACTIVE;
    CIF_Comm::status = CIF_Comm_Status::SUCCESS;
}
/* CIF_I_Comm_Startpoint::CIF_I_Comm_Starpoint() */


/*
 * CIF_I_Comm_Startpoint::activate(host, port)
 */
CIF_Bool
CIF_I_Comm_Startpoint::activate(
    ARG_IN const char *				host,
    ARG_IN unsigned short			port)
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that we are operating on a valid but inactive object
     */
    CIF_Comm_Status				status;
    
    status = l_verify_Object_State(OBJECT_STATE_INACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }
    
    /*
     * Obtain a new startpoint by attaching to a remote endpoint
     */
    CIF_Bool					rc;
    char *					url;

    url = new char[strlen(host) + 22];

    sprintf(url, "x-nexus://%s:%d/", host, port);
    if (globus_nexus_attach(url, &l_nx_sp) == GLOBUS_SUCCESS)
    {
	l_object_state = OBJECT_STATE_ACTIVE;
	CIF_Comm::status = CIF_Comm_Status::SUCCESS;
	rc = true;
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	rc = false;
    }

    delete [] url;

    return rc;
}
/* CIF_I_Comm_Startpoint::activate(host, port) */


/*
 * CIF_I_Comm_Startpoint::activate(endpoint)
 */
CIF_Bool
CIF_I_Comm_Startpoint::activate(
    ARG_IN CIF_I_Comm_Endpoint &		ep)
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that we are operating on a valid but inactive object
     */
    CIF_Comm_Status			status;
    
    status = l_verify_Object_State(OBJECT_STATE_INACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    /*
     * Create the startpoint from a local endpoint
     */
    ep.i_bind_Startpoint(&l_nx_sp);
    if (CIF_Comm::status == CIF_Comm_Status::SUCCESS)
    {
	l_object_state = OBJECT_STATE_ACTIVE;
	return true;
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return false;
    }
}
/* CIF_I_Comm_Startpoint::activate(endpoint) */


/*
 * CIF_I_Comm_Startpoint::activate(nx_buf)
 */
CIF_Bool
CIF_I_Comm_Startpoint::activate(
    ARG_INOUT globus_nexus_buffer_t &		nx_buf)
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that we are operating on a valid but inactive object
     */
    CIF_Comm_Status				status;
    
    status = l_verify_Object_State(OBJECT_STATE_INACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }
    
    /*
     * Get the startpoint out of the buffer
     */
    if (globus_nexus_get_startpoint(&nx_buf, &l_nx_sp, 1) == GLOBUS_SUCCESS)
    {
	l_object_state = OBJECT_STATE_ACTIVE;
	CIF_Comm::status = CIF_Comm_Status::SUCCESS;
	return true;
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return false;
    }
}
/* CIF_I_Comm_Startpoint::get_Internal_Startpoint() */


/*
 * CIF_I_Comm_Startpoint::send()
 */
CIF_Bool
CIF_I_Comm_Startpoint::send(
    ARG_INOUT globus_nexus_buffer_t &		nx_buf)
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that we are operating on a valid object
     */
    CIF_Comm_Status			status;
    
    status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    if (globus_nexus_startpoint_test(&l_nx_sp) != GLOBUS_SUCCESS)
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return false;
    }
    
    /*
     * Send the message
     */
    if (globus_nexus_send_rsr(&nx_buf,
			      &l_nx_sp,
			      CIF_I_Comm_Endpoint::i_handle_Message_HID,
			      GLOBUS_TRUE,
			      GLOBUS_FALSE) == GLOBUS_SUCCESS)
    {
	CIF_Comm::status = CIF_Comm_Status::SUCCESS;
	return true;
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return false;
    }
}
/* CIF_I_Comm_Startpoint::send() */


/*
 * CIF_I_Comm_Startpoint::sizeof_Startpoint()
 */
CIF_Size
CIF_I_Comm_Startpoint::sizeof_Startpoint()
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that we are operating on a valid object
     */
    CIF_Comm_Status				status;
    
    status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return 0;
    }

    /*
     * Get the sizeof the internal startpoint
     */
    CIF_Size					size;
    
    size = (CIF_Size) globus_nexus_sizeof_startpoint(&l_nx_sp, 1);
    if (size > 0)
    {
	CIF_Comm::status = CIF_Comm_Status::SUCCESS;
	return size;
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return 0;
    }
}
/* CIF_I_Comm_Startpoint::sizeof_Startpoint() */


/*
 * CIF_I_Comm_Startpoint::put_Startpoint()
 */
CIF_Bool
CIF_I_Comm_Startpoint::put_Startpoint(
    ARG_INOUT globus_nexus_buffer_t &		nx_buf)
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that we are operating on a valid object
     */
    CIF_Comm_Status				status;
    
    status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    /*
     * Transfer the startpoint into the buffer
     */
    int						rc;
    
    rc = globus_nexus_put_startpoint_transfer(&nx_buf, &l_nx_sp, 1);
    if (rc == GLOBUS_SUCCESS)
    {
	l_object_state = OBJECT_STATE_INACTIVE;
	CIF_Comm::status = CIF_Comm_Status::SUCCESS;
	return true;
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return false;
    }
}
/* CIF_I_Comm_Startpoint::put_Startpoint() */


/*
 * CIF_I_Comm_Startpoint::deactivate()
 */
CIF_Bool
CIF_I_Comm_Startpoint::deactivate()
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    return l_deactivate(monitor);
}
/* CIF_I_Comm_Startpoint::deactivate() */


/*
 * CIF_I_Comm_Startpoint::~CIF_I_Comm_Startpoint()
 */
CIF_I_Comm_Startpoint::~CIF_I_Comm_Startpoint()
{
    CIF_Comm_Status				status;
    CIF_Toolkit_Monitor				monitor(l_mutex);

    status = CIF_Comm::status;
    {
	/*
	 * First, deactivate the object
	 */
	l_deactivate(monitor);

	/*
	 * Set state to indicate that the object has been deleted
	 */
	l_object_state = OBJECT_STATE_DELETED;
    }
    CIF_Comm::status = status;
}
/* CIF_I_Comm_Startpoint::~CIF_I_Comm_Startpoint() */


/*
 * CIF_I_Comm_Startpoint::l_deactivate()
 */
CIF_Bool
CIF_I_Comm_Startpoint::l_deactivate(
    CIF_Toolkit_Monitor &			monitor)
{
    /*
     * Verify that we are operating on a valid object
     */
    CIF_Comm_Status				status;
    
    status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    /*
     * Set the object state to inactive
     */
    l_object_state = OBJECT_STATE_INACTIVE;
    
    /*
     * Destroy the startpoint, freeing resources held by it
     */
    int						rc;
    
    rc = globus_nexus_startpoint_destroy(&l_nx_sp);

    if (rc == GLOBUS_SUCCESS)
    {
	CIF_Comm::status = CIF_Comm_Status::SUCCESS;
	return true;
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return false;
    }
}
/* CIF_I_Comm_Startpoint::l_deactivate() */


/*
 * CIF_I_Comm_Startpoint::l_verify_Object_State()
 */
CIF_Comm_Status
CIF_I_Comm_Startpoint::l_verify_Object_State(
    ARG_IN Object_State			desired_state)
{
    if (l_object_state == desired_state)
    {
	return CIF_Comm_Status::SUCCESS;
    }
    
    /*
     * We aren't in the state we explected to be, so return an appropriate
     * error to the user
     */
    switch (l_object_state)
    {
      case OBJECT_STATE_DELETED:
	globus_assert(l_object_state != OBJECT_STATE_DELETED);
	return CIF_Comm_Status::OBJECT_DELETED;
	    
      case OBJECT_STATE_ACTIVE:
	return CIF_Comm_Status::OBJECT_ACTIVE;
	
      case OBJECT_STATE_INACTIVE:
	return CIF_Comm_Status::OBJECT_INACTIVE;

      default:
	return CIF_Comm_Status::INTERNAL_ERROR;
    }
}
/* CIF_I_Comm_Startpoint::l_verify_Object_State() */


