#include "CIF_Toolkit.H"
#include "CIF_I_Comm_Endpoint.H"
#include "CIF_I_Comm_Startpoint.H"

#include <stdio.h>
#include <string.h>

class Test_NSML
{
    public:
    Test_NSML(
	int					msg_cnt)
    : l_msg_cnt(msg_cnt)
    {
    }

    public:
    void
    run(
	CIF_Toolkit_Timeout &			timeout)
    {
	const int				MAX_MSG_SZ = 512;
	char					msg_buf[MAX_MSG_SZ];
	int					msg_len;

	int					cnt;
	CIF_Bool				rc;
	
	for (cnt = 0; cnt < l_msg_cnt; cnt ++)
	{
	    globus_nexus_buffer_t		nx_buf;
	    int					nx_buf_sz;
	
	    sprintf(msg_buf, "test message %d", cnt);
	    msg_len = strlen(msg_buf) + 1;

	    nx_buf_sz = globus_nexus_sizeof_int(1)
		+ globus_nexus_sizeof_char(msg_len);
	    globus_nexus_buffer_init(&nx_buf, nx_buf_sz, 0);
	    globus_nexus_put_int(&nx_buf, &msg_len, 1);
	    globus_nexus_put_char(&nx_buf, msg_buf, msg_len);

	    rc = i_sp.send(nx_buf);
	    printf("startpoint.send(): rc=%d, status=%d\n",
		   rc,
		   (int) CIF_Comm::status);
	    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	}

	rc = i_ep.enable_Event_Listener();
	printf("endpoint.enable_Event_Listener(): rc=%d, status = %d\n",
	       rc,
	       (int) CIF_Comm::status);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);

	for (cnt = 0; cnt < l_msg_cnt; cnt ++)
	{
	    globus_nexus_buffer_t		nx_buf;

	    rc = i_ep.receive(nx_buf, timeout);
	    printf("endpoint.receive(): rc=%d, status=%d\n",
		   rc,
		   (int) CIF_Comm::status);
	    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);

	    globus_nexus_get_int(&nx_buf, &msg_len, 1);
	    globus_nexus_get_char(&nx_buf, msg_buf, msg_len);
	    globus_nexus_buffer_destroy(&nx_buf);
	    
	    printf( "Message %d: %s\n", cnt, msg_buf);
	}
    }

    protected:
    CIF_I_Comm_Endpoint				i_ep;
    CIF_I_Comm_Startpoint			i_sp;

    private:
    const int					l_msg_cnt;
};


class Test_NSML_Attach
: public Test_NSML,
  private CIF_Toolkit_Event_Listener
{
    public:
    Test_NSML_Attach(
	int					msg_cnt)
    : Test_NSML(msg_cnt)
    {
	CIF_Bool				rc;
	CIF_I_Comm_Endpoint::Properties		properties;
	const char *				host;
	unsigned short				port;
	

	//properties.set_Protocol(GLOBUS_NEXUS_PROTO_TYPE_UDP);
	properties.enable_Attach();
	properties.set_Event_Listener((CIF_Toolkit_Event_Listener *) this);

	rc = i_ep.activate(properties);
	printf("endpoint.activate(): rc=%d, status = %d\n",
	       rc,
	       (int) CIF_Comm::status);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);

	host = i_ep.get_Attach_Hostname();
	port = i_ep.get_Attach_Port();
	
	printf("ep.get_Attach_info(): host=%s, port=%hu\n", host, port);
	
	rc = i_sp.activate(host, port);
	printf("startpoint.activate(): rc=%d, status = %d\n",
	       rc,
	       (int) CIF_Comm::status);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    }

    public:
    ~Test_NSML_Attach()
    {
	CIF_Bool				rc;
	
	rc = i_sp.deactivate();
	printf("startpoint.deactivate(): rc=%d, status = %d\n",
	       rc,
	       (int) CIF_Comm::status);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);

    	rc = i_ep.deactivate();
	printf("endpoint.deactivate(): rc=%d, status = %d\n",
	       rc,
	       (int) CIF_Comm::status);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    }

    private:
    void
    handle_Event(
	CIF_Toolkit_Event &			event)
    {
	printf("handle_Event called\n");
    }
};


extern "C" int _nx_debug_level;


int    
main(
    int					argc,
    char *				argv[])
{
    const int				MSG_CNT = 4;

    CIF_Comm::activate();
    {
	_nx_debug_level = 0;
    
	CIF_Toolkit_Timeout		timeout;

	timeout = CIF_Toolkit_Timeout::INFINITY;
	Test_NSML_Attach(MSG_CNT).run(timeout);
    }
    CIF_Comm::deactivate();

    exit(0);
}
